==> open-xchange-core_7.6.3.orig.tar.bz2: open-xchange-core-7.6.3/com.openexchange.server/src/com/openexchange/tools/HostList.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.tools;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import com.google.common.collect.ImmutableSet;
import com.openexchange.java.IPAddressUtil;
import com.openexchange.java.Strings;
import com.openexchange.mail.config.IPRange;

/**
 * {@link HostList} - Represents a list of host names.
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 * @since v7.8.2
 */
public class HostList {

    /**
     * The empty host list.
     */
    public static final HostList EMPTY = new HostList(Collections.<IPRange> emptyList(), Collections.<String> emptySet(), Collections.<String> emptySet(), "");

    /**
     * Accepts a comma-separated list of IP addresses, IP address ranges, and host names.
     *
     * @param hostList The host list to parse
     * @return The resulting {@link HostList} instance
     * @throws IllegalArgumentException If parsing fails
     */
    public static HostList valueOf(String hostList) {
        if (Strings.isEmpty(hostList)) {
            return EMPTY;
        }

        String[] tokens = Strings.splitByComma(hostList);
        Set<String> matchingHostNames = new HashSet<String>(tokens.length);
        Set<String> matchingAppendixHostNames = new HashSet<String>(tokens.length);
        List<IPRange> ipRanges = new ArrayList<IPRange>(tokens.length);
        for (String token : tokens) {
            if (Strings.isNotEmpty(token)) {
                token = Strings.asciiLowerCase(token);
                boolean isIp = false;
                try {
                    ipRanges.add(IPRange.parseRange(token));
                    isIp = true;
                } catch (IllegalArgumentException e) {
                    // Apparently no IP address
                }

                if (false == isIp) {
                    if (token.startsWith("*")) {
                        // Wild-card host name; e.g. "*.open-xchange.com"
                        String appendixToken = token.substring(1);
                        if (Strings.isEmpty(appendixToken) || appendixToken.indexOf('*') >= 0) {
                            throw new IllegalArgumentException("Invalid wild-card host name: " + token);
                        }
                        matchingAppendixHostNames.add(appendixToken);
                    } else {
                        if (token.indexOf('*') > 0) {
                            // Wild-card only allowed at first position
                            throw new IllegalArgumentException("Invalid wild-card host name: " + token);
                        }
                        // Exact match
                        matchingHostNames.add(token);
                    }
                }
            }
        }

        return new HostList(ipRanges, matchingAppendixHostNames, matchingHostNames, hostList);
    }

    // ----------------------------------------------------------------------------------------------------------------------------------

    private final List<IPRange> ipRanges;
    private final Set<String> matchingAppendixHostNames;
    private final Set<String> matchingHostNames;
    private final String hostString;

    /**
     * Initializes a new {@link HostList}.
     */
    private HostList(List<IPRange> ipRanges, Set<String> matchingAppendixHostNames, Set<String> matchingHostNames, String hostString) {
        super();
        this.ipRanges = ipRanges;
        this.hostString = hostString;
        this.matchingAppendixHostNames = ImmutableSet.copyOf(matchingAppendixHostNames);
        this.matchingHostNames = ImmutableSet.copyOf(matchingHostNames);
    }

    private final static int INADDR4SZ = 4;
    private final static int INADDR16SZ = 16;
    private final static int INT16SZ = 2;

    static String numericToTextFormatV4(byte[] src) {
        StringBuilder sb = new StringBuilder(16);
        sb.append((src[0] & 0xff)).append('.');
        sb.append((src[1] & 0xff)).append('.');
        sb.append((src[2] & 0xff)).append('.');
        sb.append((src[3] & 0xff));
        return sb.toString();
    }

    static String numericToTextFormatV6(byte[] src) {
        StringBuffer sb = new StringBuffer(39);
        for (int i = 0; i < (INADDR16SZ / INT16SZ); i++) {
            sb.append(Integer.toHexString(((src[i << 1] << 8) & 0xff00) | (src[(i << 1) + 1] & 0xff)));
            if (i < (INADDR16SZ / INT16SZ) - 1) {
                sb.append(":");
            }
        }
        return sb.toString();
    }

    /**
     * Checks if this host list is empty.
     *
     * @return <code>true</code> if empty; otherwise <code>false</code>
     */
    public boolean isEmpty() {
        return ipRanges.isEmpty() && matchingAppendixHostNames.isEmpty() && matchingHostNames.isEmpty();
    }

    /**
     * Gets the host string from which this instance was parsed.
     *
     * @return The host string
     */
    public String getHostString() {
        return hostString;
    }

    /**
     * Checks if specified host name is contained in this host list.
     * <p>
     * The host name can either be a machine name, such as "<code>java.sun.com</code>", or a textual representation of its IP address.
     *
     * @param hostAddress The host address
     * @return <code>true</code> if contained; otherwise <code>false</code>
     */
    public boolean contains(InetAddress hostAddress) {
        return contains(hostAddress, true);
    }

    private boolean contains(InetAddress hostAddress, boolean checkHostName) {
        if (null == hostAddress) {
            return false;
        }

        byte[] octets = hostAddress.getAddress();
        if (null != octets) {
            if (INADDR4SZ == octets.length) {
                // IPv4
                for (IPRange ipRange : this.ipRanges) {
                    if (ipRange.containsIPv4(octets, numericToTextFormatV4(octets))) {
                        return true;
                    }
                }
            } else if (INADDR16SZ == octets.length) {
                // IPv6
                for (IPRange ipRange : this.ipRanges) {
                    if (ipRange.containsIPv6(octets, numericToTextFormatV6(octets))) {
                        return true;
                    }
                }
            }
        }

        return checkHostName ? contains(hostAddress.getHostName()) : false;
    }

    /**
     * Checks if specified host name is contained in this host list.
     * <p>
     * The host name can either be a machine name, such as "<code>java.sun.com</code>", or a textual representation of its IP address.
     *
     * @param hostName The host name; either a machine name or a textual representation of its IP address
     * @return <code>true</code> if contained; otherwise <code>false</code>
     */
    public boolean contains(String hostName) {
        if (Strings.isEmpty(hostName)) {
            return false;
        }

        String toCheck = Strings.asciiLowerCase(hostName);

        // Test for IP address
        byte[] octets = IPAddressUtil.textToNumericFormatV4(toCheck);
        if (octets != null) {
            // IPv4
            for (IPRange ipRange : this.ipRanges) {
                if (ipRange.containsIPv4(octets, toCheck)) {
                    return true;
                }
            }
        }

        octets = IPAddressUtil.textToNumericFormatV6(toCheck);
        if (octets != null) {
            if (octets.length == 4) {
                // IPv4 mapped IPv6 address
                for (IPRange ipRange : this.ipRanges) {
                    if (ipRange.containsIPv4(octets, toCheck)) {
                        return true;
                    }
                }
            }

            // IPv6
            for (IPRange ipRange : this.ipRanges) {
                if (ipRange.containsIPv6(octets, toCheck)) {
                    return true;
                }
            }
        }

        if (this.matchingAppendixHostNames != null) {
            for (String appendixHostName : this.matchingAppendixHostNames) {
                if (toCheck.endsWith(appendixHostName)) {
                    return true;
                }
            }
        }

        if (this.matchingHostNames.contains(toCheck)) {
            return true;
        }

        // Need to resolve as last resort
        try {
            return contains(InetAddress.getByName(toCheck), false);
        } catch (UnknownHostException e) {
            // Cannot be resolved
            return false;
        }
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder(32);
        builder.append("[");
        if (this.ipRanges != null) {
            builder.append("ipRanges=").append(this.ipRanges).append(", ");
        }
        if (this.matchingAppendixHostNames != null) {
            builder.append("wild-card_hostNames=[");
            Iterator<String> it = this.matchingAppendixHostNames.iterator();
            if (it.hasNext()) {
                builder.append('*').append(it.next());
                while (it.hasNext()) {
                    builder.append(", ").append('*').append(it.next());
                }
            }
            builder.append("], ");
        }
        if (this.matchingHostNames != null) {
            builder.append("hostNames=").append(this.matchingHostNames);
        }
        builder.append("]");
        return builder.toString();
    }

}

==> open-xchange-core_7.8.1.orig.tar.bz2: open-xchange-core-7.8.1/com.openexchange.server/src/com/openexchange/tools/HostList.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.tools;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import com.openexchange.java.IPAddressUtil;
import com.openexchange.java.Strings;
import com.openexchange.mail.config.IPRange;

/**
 * {@link HostList} - Represents a list of host names.
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 * @since v7.8.2
 */
public class HostList {

    /**
     * The empty host list.
     */
    public static final HostList EMPTY = new HostList(Collections.<IPRange> emptyList(), Collections.<String> emptySet(), Collections.<String> emptySet());

    /**
     * Accepts a comma-separated list of IP addresses, IP address ranges, and host names.
     *
     * @param hostList The host list to parse
     * @return The resulting {@link HostList} instance
     * @throws IllegalArgumentException If parsing fails
     */
    public static HostList valueOf(String hostList) {
        if (Strings.isEmpty(hostList)) {
            return EMPTY;
        }

        String[] tokens = Strings.splitByComma(hostList);
        Set<String> matchingHostNames = new HashSet<String>(tokens.length);
        Set<String> matchingAppendixHostNames = new HashSet<String>(tokens.length);
        List<IPRange> ipRanges = new ArrayList<IPRange>(tokens.length);
        for (String token : tokens) {
            if (false == Strings.isEmpty(token)) {
                token = Strings.asciiLowerCase(token);
                boolean isIp = false;
                try {
                    ipRanges.add(IPRange.parseRange(token));
                    isIp = true;
                } catch (IllegalArgumentException e) {
                    // Apparently no IP address
                }

                if (false == isIp) {
                    if (token.startsWith("*")) {
                        // Wild-card host name; e.g. "*.open-xchange.com"
                        String appendixToken = token.substring(1);
                        if (Strings.isEmpty(appendixToken) || appendixToken.indexOf('*') >= 0) {
                            throw new IllegalArgumentException("Invalid wild-card host name: " + token);
                        }
                        matchingAppendixHostNames.add(appendixToken);
                    } else {
                        if (token.indexOf('*') > 0) {
                            // Wild-card only allowed at first position
                            throw new IllegalArgumentException("Invalid wild-card host name: " + token);
                        }
                        // Exact match
                        matchingHostNames.add(token);
                    }
                }
            }
        }

        return new HostList(ipRanges, matchingAppendixHostNames, matchingHostNames);
    }

    // ----------------------------------------------------------------------------------------------------------------------------------

    private final List<IPRange> ipRanges;
    private final Set<String> matchingAppendixHostNames;
    private final Set<String> matchingHostNames;

    /**
     * Initializes a new {@link HostList}.
     */
    private HostList(List<IPRange> ipRanges, Set<String> matchingAppendixHostNames, Set<String> matchingHostNames) {
        super();
        this.ipRanges = ipRanges;
        this.matchingAppendixHostNames = matchingAppendixHostNames.isEmpty() ? null : matchingAppendixHostNames;
        this.matchingHostNames = matchingHostNames;
    }

    /**
     * Checks if specified host name is contained in this host list.
     * <p>
     * The host name can either be a machine name, such as "<code>java.sun.com</code>", or a textual representation of its IP address.
     *
     * @param hostName The host name; either a machine name or a textual representation of its IP address
     * @return <code>true</code> if contained; otherwise <code>false</code>
     */
    public boolean contains(String hostName) {
        if (Strings.isEmpty(hostName)) {
            return false;
        }

        String toCheck = Strings.asciiLowerCase(hostName);

        // Test for IP address
        byte[] octets = IPAddressUtil.textToNumericFormatV4(toCheck);
        if (null != octets) {
            // IPv4
            for (IPRange ipRange : ipRanges) {
                if (ipRange.containsIPv4(octets, toCheck)) {
                    return true;
                }
            }
        }

        octets = IPAddressUtil.textToNumericFormatV6(toCheck);
        if (null != octets) {
            // IPv6
            for (IPRange ipRange : ipRanges) {
                if (ipRange.containsIPv6(octets, toCheck)) {
                    return true;
                }
            }
        }

        if (null != matchingAppendixHostNames) {
            for (String appendixHostName : matchingAppendixHostNames) {
                if (toCheck.endsWith(appendixHostName)) {
                    return true;
                }
            }
        }

        return matchingHostNames.contains(Strings.asciiLowerCase(toCheck));
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder(32);
        builder.append("[");
        if (ipRanges != null) {
            builder.append("ipRanges=").append(ipRanges).append(", ");
        }
        if (matchingAppendixHostNames != null) {
            builder.append("wild-card_hostNames=[");
            Iterator<String> it = matchingAppendixHostNames.iterator();
            if (it.hasNext()) {
                builder.append('*').append(it.next());
                while (it.hasNext()) {
                    builder.append(", ").append('*').append(it.next());
                }
            }
            builder.append("], ");
        }
        if (matchingHostNames != null) {
            builder.append("hostNames=").append(matchingHostNames);
        }
        builder.append("]");
        return builder.toString();
    }

}
